function sos = sosineq(sos,symexpr,info)
% SOSINEQ --- Add a new inequality constraint f(x) >= 0
%    to an SOS program 
%
% SOSP = sosineq(SOSP,EXPR)
%
% SOSP is the sum of squares program.
% EXPR is the symbolic expression in the left hand side of 
%     the constraint, i.e., f(x).
%
% EXPR can be a column vector. In this case, several inequality
% constraints will be added simultaneously to the sum of
% squares program.
%
% Alternatively, SOSP = sosineq(SOSP,EXPR,INFO) may be used.
% The third argument INFO describes one of the following things:
%
% -- the interval of x where f(x) has to be non-negative. Currently,
%    this is possible only if the sum of squares program is univariate.
%    For example,
%    INFO = [a b] will make f(x) >= 0 for a <= x <= b.
%    INFO = [-Inf b] will make f(x) >= 0 for -Inf < x <= b.
%    INFO = [a Inf] will make f(x) >= 0 for a <= x < Inf.
%
% -- whether or not sparsity will be utilized in the sum of squares
%    decomposition (by default it will not be utilized). To make
%    use of the fact that a polynomial is sparse, give INFO = 'sparse'
%    as the third argument to SOSINEQ.
%

% This file is part of SOSTOOLS - Sum of Squares Toolbox ver 1.00.
%
% Copyright (C) 2002  S. Prajna (1), A. Papachristodoulou (1), P. A. Parrilo (2)
% (1) Control and Dynamical Systems - California Institute of Technology,
%     Pasadena, CA 91125, USA.
% (2) Institut fur Automatik - ETH Zurich, CH-8092 Zurich, Switzerland.
%
% Send feedback to: sostools@cds.caltech.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%


% 03/20/02 - SP -- Interval info

% Interval
if nargin > 2 & isnumeric(info)
    if length(sos.symvartable) ~= 1
        error('Not a univariate polynomial.');
    end;
    for i = 1:size(symexpr,1)
        symexpr(i) = preprocess(symexpr(i),sos.symvartable,info);
        sos = sosconstr(sos,'ineq',symexpr(i));
        sos.expr.type{sos.expr.num} = 'posint';
    end;
    return;
end;

% Sparse polynomial
if nargin > 2 & strcmp(info,'sparse');
   for i = 1:size(symexpr,1)
       sos = sosconstr(sos,'ineq',symexpr(i));
       sos.expr.type{sos.expr.num} = 'sparse';
   end;
   return;
end;

% No option selected
for i = 1:size(symexpr,1)
    sos = sosconstr(sos,'ineq',symexpr(i,:));
end;


% ============================================================
function newsymexpr = preprocess(symexpr,var,info)

% Get the maximum degree of the independent variable
maxdeg = 0;
dummy = diff(symexpr,var);
while dummy ~= 0
    maxdeg = maxdeg+1;
    dummy = diff(dummy,var);
end;

% Substitute var
if info(2)==Inf
    newvar = var+info(1);
    newsymexpr = subs(symexpr,var,newvar);
elseif info(1)==-Inf
    newvar = -var+info(2);
    newsymexpr = subs(symexpr,var,newvar);
else
    newvar = (info(2)-info(1))/2*(1-var)/(1+var) + (info(2)+info(1))/2;
    newsymexpr = subs(symexpr,var,newvar)*(1+var).^maxdeg;
end;
